@@ -1,5 +1,4 @@
 import os
-import shutil
 import sys
 import unittest
 import warnings
@@ -7,7 +6,6 @@ import h5py
 import numpy as np
 import meep as mp
 
-
 try:
     unicode
 except NameError:
@@ -21,6 +19,14 @@ class TestSimulation(unittest.TestCase):
     def setUp(self):
         print("Running {}".format(self._testMethodName))
 
+    @classmethod
+    def setUpClass(cls):
+        cls.temp_dir = mp.make_output_directory()
+
+    @classmethod
+    def tearDownClass(cls):
+        mp.delete_directory(cls.temp_dir)
+
     def test_interpolate_numbers(self):
 
         expected = [
@@ -121,39 +127,20 @@ class TestSimulation(unittest.TestCase):
 
     def test_use_output_directory_default(self):
         sim = self.init_simple_simulation()
-        sim.use_output_directory()
+        output_dir = os.path.join(self.temp_dir, 'simulation-out')
+        sim.use_output_directory(output_dir)
         sim.run(mp.at_end(mp.output_efield_z), until=200)
 
-        output_dir = 'simulation-out'
         self.assertTrue(os.path.exists(os.path.join(output_dir, self.fname)))
 
-        mp.all_wait()
-        if mp.am_master():
-            shutil.rmtree(output_dir)
-
-    def test_use_output_directory_custom(self):
-        sim = self.init_simple_simulation()
-        sim.use_output_directory('custom_dir')
-        sim.run(mp.at_end(mp.output_efield_z), until=200)
-
-        output_dir = 'custom_dir'
-        self.assertTrue(os.path.exists(os.path.join(output_dir, self.fname)))
-
-        mp.all_wait()
-        if mp.am_master():
-            shutil.rmtree(output_dir)
-
     def test_at_time(self):
         sim = self.init_simple_simulation()
+        sim.use_output_directory(self.temp_dir)
         sim.run(mp.at_time(100, mp.output_efield_z), until=200)
 
-        fname = 'simulation-ez-000100.00.h5'
+        fname = os.path.join(self.temp_dir, 'simulation-ez-000100.00.h5')
         self.assertTrue(os.path.exists(fname))
 
-        mp.all_wait()
-        if mp.am_master():
-            os.remove(fname)
-
     def test_after_sources_and_time(self):
         sim = self.init_simple_simulation()
 
@@ -168,15 +155,12 @@ class TestSimulation(unittest.TestCase):
 
     def test_with_prefix(self):
         sim = self.init_simple_simulation()
+        sim.use_output_directory(self.temp_dir)
         sim.run(mp.with_prefix('test_prefix-', mp.at_end(mp.output_efield_z)), until=200)
 
-        fname = 'test_prefix-simulation-ez-000200.00.h5'
+        fname = os.path.join(self.temp_dir, 'test_prefix-simulation-ez-000200.00.h5')
         self.assertTrue(os.path.exists(fname))
 
-        mp.all_wait()
-        if mp.am_master():
-            os.remove(fname)
-
     def test_extra_materials(self):
         sim = self.init_simple_simulation()
         sim.extra_materials = [mp.Medium(epsilon=5), mp.Medium(epsilon=10)]
@@ -199,26 +183,26 @@ class TestSimulation(unittest.TestCase):
 
     def test_in_volume(self):
         sim = self.init_simple_simulation()
+        sim.use_output_directory(self.temp_dir)
         sim.filename_prefix = 'test_in_volume'
         vol = mp.Volume(mp.Vector3(), size=mp.Vector3(x=2))
         sim.run(mp.at_end(mp.in_volume(vol, mp.output_efield_z)), until=200)
-
-    def test_in_point(self):
-        sim = self.init_simple_simulation(filename_prefix='test_in_point')
-        fn = sim.filename_prefix + '-ez-000200.00.h5'
-        pt = mp.Vector3()
-        sim.run(mp.at_end(mp.in_point(pt, mp.output_efield_z)), until=200)
+        fn = os.path.join(self.temp_dir, 'test_in_volume-ez-000200.00.h5')
         self.assertTrue(os.path.exists(fn))
 
-        mp.all_wait()
-        if mp.am_master():
-            os.remove(fn)
+    def test_in_point(self):
+        sim = self.init_simple_simulation()
+        sim.use_output_directory(self.temp_dir)
+        sim.filename_prefix = 'test_in_point'
+        pt = mp.Vector3()
+        sim.run(mp.at_end(mp.in_point(pt, mp.output_efield_z)), until=200)
+        fn = os.path.join(self.temp_dir, 'test_in_point-ez-000200.00.h5')
+        self.assertTrue(os.path.exists(fn))
 
     def test_epsilon_input_file(self):
         sim = self.init_simple_simulation()
         eps_input_fname = 'cyl-ellipsoid-eps-ref.h5'
-        eps_input_dir = os.path.join(os.path.abspath(os.path.realpath(os.path.dirname(__file__))),
-                                     '..', '..', 'libmeepgeom')
+        eps_input_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', '..', 'tests'))
         eps_input_path = os.path.join(eps_input_dir, eps_input_fname)
         sim.epsilon_input_file = eps_input_path
 
@@ -237,12 +221,11 @@ class TestSimulation(unittest.TestCase):
     def test_numpy_epsilon(self):
         sim = self.init_simple_simulation()
         eps_input_fname = 'cyl-ellipsoid-eps-ref.h5'
-        eps_input_dir = os.path.join(os.path.abspath(os.path.realpath(os.path.dirname(__file__))),
-                                     '..', '..', 'libmeepgeom')
+        eps_input_dir = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..', '..', 'tests'))
         eps_input_path = os.path.join(eps_input_dir, eps_input_fname)
 
         with h5py.File(eps_input_path, 'r') as f:
-            sim.default_material = f['eps'].value
+            sim.default_material = f['eps'][()]
 
         sim.run(until=200)
         fp = sim.get_field_point(mp.Ez, mp.Vector3(x=1))
@@ -274,12 +257,10 @@ class TestSimulation(unittest.TestCase):
         eps = {'arr1': None, 'arr2': None}
 
         def get_arr1(sim):
-            eps['arr1'] = sim.get_array(mp.Volume(mp.Vector3(), mp.Vector3(10, 10)),
-                                        component=mp.Dielectric)
+            eps['arr1'] = sim.get_array(mp.Dielectric, mp.Volume(mp.Vector3(), mp.Vector3(10, 10)))
 
         def get_arr2(sim):
-            eps['arr2'] = sim.get_array(mp.Volume(mp.Vector3(), mp.Vector3(10, 10)),
-                                        component=mp.Dielectric)
+            eps['arr2'] = sim.get_array(mp.Dielectric, mp.Volume(mp.Vector3(), mp.Vector3(10, 10)))
 
         sim.run(mp.at_time(50, get_arr1), mp.at_time(100, change_geom),
                 mp.at_end(get_arr2), until=200)
@@ -313,21 +294,24 @@ class TestSimulation(unittest.TestCase):
         sim.field_energy_in_box(tv)
         sim.field_energy_in_box(v)
 
-    def test_load_dump_structure(self):
+    def _load_dump_structure(self, chunk_file=False, chunk_sim=False):
         from meep.materials import Al
         resolution = 50
         cell = mp.Vector3(5, 5)
         sources = mp.Source(src=mp.GaussianSource(1, fwidth=0.2), center=mp.Vector3(), component=mp.Ez)
         one_by_one = mp.Vector3(1, 1, mp.inf)
-        geometry = [mp.Block(material=Al, center=mp.Vector3(-1.5, -1.5), size=one_by_one),
-                    mp.Block(material=mp.Medium(epsilon=13), center=mp.Vector3(1.5, 1.5), size=one_by_one)]
+        geometry = [mp.Block(material=Al, center=mp.Vector3(), size=one_by_one),
+                    mp.Block(material=mp.Medium(epsilon=13), center=mp.Vector3(1), size=one_by_one)]
         pml_layers = [mp.PML(0.5)]
 
-        sim = mp.Simulation(resolution=resolution,
-                            cell_size=cell,
-                            boundary_layers=pml_layers,
-                            geometry=geometry,
-                            sources=[sources])
+        symmetries = [mp.Mirror(mp.Y)]
+
+        sim1 = mp.Simulation(resolution=resolution,
+                             cell_size=cell,
+                             boundary_layers=pml_layers,
+                             geometry=geometry,
+                             symmetries=symmetries,
+                             sources=[sources])
 
         sample_point = mp.Vector3(0.12, -0.29)
         ref_field_points = []
@@ -336,14 +320,24 @@ class TestSimulation(unittest.TestCase):
             p = sim.get_field_point(mp.Ez, sample_point)
             ref_field_points.append(p.real)
 
-        sim.run(mp.at_every(5, get_ref_field_point), until=50)
-        dump_fn = 'test_load_dump_structure.h5'
-        sim.dump_structure(dump_fn)
+        sim1.run(mp.at_every(5, get_ref_field_point), until=50)
+        dump_fn = os.path.join(self.temp_dir, 'test_load_dump_structure.h5')
+        dump_chunk_fname = None
+        chunk_layout = None
+        sim1.dump_structure(dump_fn)
+        if chunk_file:
+            dump_chunk_fname = os.path.join(self.temp_dir, 'test_load_dump_structure_chunks.h5')
+            sim1.dump_chunk_layout(dump_chunk_fname)
+            chunk_layout = dump_chunk_fname
+        if chunk_sim:
+            chunk_layout = sim1
 
         sim = mp.Simulation(resolution=resolution,
                             cell_size=cell,
                             boundary_layers=pml_layers,
                             sources=[sources],
+                            symmetries=symmetries,
+                            chunk_layout=chunk_layout,
                             load_structure=dump_fn)
 
         field_points = []
@@ -357,12 +351,18 @@ class TestSimulation(unittest.TestCase):
         for ref_pt, pt in zip(ref_field_points, field_points):
             self.assertAlmostEqual(ref_pt, pt)
 
-        mp.all_wait()
-        if mp.am_master():
-            os.remove(dump_fn)
+    def test_load_dump_structure(self):
+        self._load_dump_structure()
+
+    def test_load_dump_chunk_layout_file(self):
+        self._load_dump_structure(chunk_file=True)
+
+    def test_load_dump_chunk_layout_sim(self):
+        self._load_dump_structure(chunk_sim=True)
 
     def test_get_array_output(self):
         sim = self.init_simple_simulation()
+        sim.use_output_directory(self.temp_dir)
         sim.symmetries = []
         sim.geometry = [mp.Cylinder(0.2, material=mp.Medium(index=3))]
         sim.filename_prefix = 'test_get_array_output'
@@ -373,26 +373,26 @@ class TestSimulation(unittest.TestCase):
         mp.output_tot_pwr(sim)
         mp.output_efield(sim)
 
-        eps_arr = sim.get_epsilon()
-        efield_z_arr = sim.get_efield_z()
-        energy_arr = sim.get_tot_pwr()
-        efield_arr = sim.get_efield()
+        eps_arr = sim.get_epsilon(snap=True)
+        efield_z_arr = sim.get_efield_z(snap=True)
+        energy_arr = sim.get_tot_pwr(snap=True)
+        efield_arr = sim.get_efield(snap=True)
 
-        fname_fmt = "test_get_array_output-{}-000020.00.h5"
+        fname_fmt = os.path.join(self.temp_dir, 'test_get_array_output-{}-000020.00.h5')
 
         with h5py.File(fname_fmt.format('eps'), 'r') as f:
-            eps = f['eps'].value
+            eps = f['eps'][()]
 
         with h5py.File(fname_fmt.format('ez'), 'r') as f:
-            efield_z = f['ez'].value
+            efield_z = f['ez'][()]
 
         with h5py.File(fname_fmt.format('energy'), 'r') as f:
-            energy = f['energy'].value
+            energy = f['energy'][()]
 
         with h5py.File(fname_fmt.format('e'), 'r') as f:
-            ex = f['ex'].value
-            ey = f['ey'].value
-            ez = f['ez'].value
+            ex = f['ex'][()]
+            ey = f['ey'][()]
+            ez = f['ez'][()]
             efield = np.stack([ex, ey, ez], axis=-1)
 
         np.testing.assert_allclose(eps, eps_arr)
@@ -423,6 +423,7 @@ class TestSimulation(unittest.TestCase):
             resolution=resolution
         )
 
+        sim.use_output_directory(self.temp_dir)
         sim.run(mp.synchronized_magnetic(mp.output_bfield_y), until=10)
 
     def test_harminv_warnings(self):
@@ -486,7 +487,7 @@ class TestSimulation(unittest.TestCase):
             assert_two(v2)
             v3 = mp.vec(three)
             assert_three(v3)
-            assert_raises(four, NotImplementedError)
+            assert_raises(four, (NotImplementedError,TypeError))
 
         check_iterable([1], [1, 2], [1, 2, 3], [1, 2, 3, 4])
         check_iterable((1,), (1, 2), (1, 2, 3), (1, 2, 3, 4))
@@ -574,6 +575,69 @@ class TestSimulation(unittest.TestCase):
 
         self.assertAlmostEqual(result[0], -0.0599602798684155)
 
+    def test_source_slice(self):
+        sim = self.init_simple_simulation()
+        sim.run(until=1)
+
+        vol1d = mp.Volume(center=mp.Vector3(0.1234,0), size=mp.Vector3(0,5.07))
+        source_slice = sim.get_source(mp.Ez, vol=vol1d)
+        x,y,z,w = sim.get_array_metadata(vol=vol1d)
+        self.assertEqual(source_slice.shape, w.shape)
+        self.assertEqual(np.sum(source_slice), 0)
+
+        vol2d = mp.Volume(center=mp.Vector3(-0.541,0.791), size=mp.Vector3(3.5,2.8))
+        source_slice = sim.get_source(mp.Ez, vol=vol2d)
+        x,y,z,w = sim.get_array_metadata(vol=vol2d)
+        self.assertEqual(source_slice.shape, w.shape)
+        self.assertNotEqual(np.sum(source_slice), 0)
+
+    def test_has_mu(self):
+
+        def _check(med, expected, default=mp.Medium()):
+            geometry = [mp.Block(center=mp.Vector3(), size=mp.Vector3(1, 1), material=med)]
+            sim = mp.Simulation(cell_size=mp.Vector3(5, 5), resolution=10, geometry=geometry,
+                                default_material=default)
+
+            result = sim.has_mu()
+            if expected:
+                self.assertTrue(result)
+            else:
+                self.assertFalse(result)
+
+            print("Estimated memory usage: {}".format(sim.get_estimated_memory_usage()))
+
+        def mat_func(p):
+            return mp.Medium()
+
+        _check(mp.Medium(mu_diag=mp.Vector3(2, 1, 1)), True)
+        _check(mp.Medium(mu_offdiag=mp.Vector3(0.1, 0.2, 0.3)), True)
+        _check(mp.Medium(), True, mp.Medium(mu_diag=mp.Vector3(1, 1, 1.1)))
+        _check(mp.Medium(), False)
+        _check(mat_func, False)
+
+    def test_iterable_as_v3(self):
+        t0 = ()
+        t1 = (1,)
+        t2 = (1, 2)
+        t3 = (1, 2, 3)
+        l0 = []
+        l1 = [1]
+        l2 = [1, 2]
+        l3 = [1, 2, 3]
+        v0 = mp.Vector3()
+        v1 = mp.Vector3(1)
+        v2 = mp.Vector3(1, 2)
+        v3 = mp.Vector3(1, 2, 3)
+
+        sim = self.init_simple_simulation()
+        sim.run(until=1)
+
+        for t, l, v3 in zip([t0, t1, t2, t3], [l0, l1, l2, l3], [v0, v1, v2, v3]):
+            pt1 = sim.get_field_point(mp.Ez, t)
+            pt2 = sim.get_field_point(mp.Ez, l)
+            expected = sim.get_field_point(mp.Ez, v3)
+            self.assertAlmostEqual(pt1, expected)
+            self.assertAlmostEqual(pt2, expected)
 
 if __name__ == '__main__':
     unittest.main()
