@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2019 Massachusetts Institute of Technology
+/* Copyright (C) 2005-2021 Massachusetts Institute of Technology
 %
 %  This program is free software; you can redistribute it and/or modify
 %  it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ fields::fields(structure *s, double m, double beta, bool zero_fields_near_cylori
   typedef fields_chunk *fields_chunk_ptr;
   chunks = new fields_chunk_ptr[num_chunks];
   for (int i = 0; i < num_chunks; i++)
-    chunks[i] = new fields_chunk(s->chunks[i], outdir, m, beta, zero_fields_near_cylorigin);
+    chunks[i] = new fields_chunk(s->chunks[i], outdir, m, beta, zero_fields_near_cylorigin, i);
   FOR_FIELD_TYPES(ft) {
     for (int ip = 0; ip < 3; ip++) {
       comm_sizes[ft][ip] = new size_t[num_chunks * num_chunks];
@@ -111,7 +111,7 @@ fields::fields(const fields &thef)
   typedef fields_chunk *fields_chunk_ptr;
   chunks = new fields_chunk_ptr[num_chunks];
   for (int i = 0; i < num_chunks; i++)
-    chunks[i] = new fields_chunk(*thef.chunks[i]);
+    chunks[i] = new fields_chunk(*thef.chunks[i], i);
   FOR_FIELD_TYPES(ft) {
     for (int ip = 0; ip < 3; ip++) {
       comm_sizes[ft][ip] = new size_t[num_chunks * num_chunks];
@@ -208,10 +208,11 @@ fields_chunk::~fields_chunk() {
 }
 
 fields_chunk::fields_chunk(structure_chunk *the_s, const char *od, double m, double beta,
-                           bool zero_fields_near_cylorigin)
+                           bool zero_fields_near_cylorigin, int chunkidx)
     : gv(the_s->gv), v(the_s->v), m(m), zero_fields_near_cylorigin(zero_fields_near_cylorigin),
       beta(beta) {
   s = the_s;
+  chunk_idx = chunkidx;
   s->refcount++;
   outdir = od;
   new_s = NULL;
@@ -267,7 +268,8 @@ fields_chunk::fields_chunk(structure_chunk *the_s, const char *od, double m, dou
   figure_out_step_plan();
 }
 
-fields_chunk::fields_chunk(const fields_chunk &thef) : gv(thef.gv), v(thef.v) {
+fields_chunk::fields_chunk(const fields_chunk &thef, int chunkidx) : gv(thef.gv), v(thef.v) {
+  chunk_idx = chunkidx;
   s = thef.s;
   s->refcount++;
   outdir = thef.outdir;
@@ -488,7 +490,9 @@ void fields::require_component(component c) {
           chunks[i]->s->has_chi(Hx, Z) || chunks[i]->s->has_chi(Hy, Z) ||
           chunks[i]->s->has_chi(Hz, X) || chunks[i]->s->has_chi(Hz, Y))
         break;
+    am_now_working_on(MpiTime);
     aniso2d = or_to_all(i < num_chunks);
+    finished_working();
   }
   if (aniso2d && beta != 0 && is_real)
     abort("Nonzero beta need complex fields when mu/epsilon couple TE and TM");
@@ -503,7 +507,9 @@ void fields::require_component(component c) {
   }
 
   if (need_to_reconnect) figure_out_step_plan();
+  am_now_working_on(MpiTime);
   if (sum_to_all(need_to_reconnect)) chunk_connections_valid = false;
+  finished_working();
 }
 
 void fields_chunk::remove_sources() {
@@ -654,6 +660,9 @@ void fields::unset_solve_cw_omega() {
     chunks[i]->unset_solve_cw_omega();
 }
 
+/* implement mirror boundary conditions for i outside 0..n-1: */
+static int mirrorindex(int i, int n) { return i >= n ? 2 * n - 1 - i : (i < 0 ? -1 - i : i); }
+
 /* Linearly interpolate a given point in a 3d grid of data.  The point
    coordinates should be in the range [0,1], or at the very least [-1,2]
    ... anything outside [0,1] is *mirror* reflected into [0,1] */
@@ -664,34 +673,24 @@ realnum linear_interpolate(realnum rx, realnum ry, realnum rz, realnum *data, in
   realnum dx, dy, dz;
 
   /* mirror boundary conditions for r just beyond the boundary */
-  if (rx < 0.0)
-    rx = -rx;
-  else if (rx > 1.0)
-    rx = 1.0 - rx;
-  if (ry < 0.0)
-    ry = -ry;
-  else if (ry > 1.0)
-    ry = 1.0 - ry;
-  if (rz < 0.0)
-    rz = -rz;
-  else if (rz > 1.0)
-    rz = 1.0 - rz;
+  rx = rx < 0.0 ? -rx : (rx > 1.0 ? 1.0 - rx : rx);
+  ry = ry < 0.0 ? -ry : (ry > 1.0 ? 1.0 - ry : ry);
+  rz = rz < 0.0 ? -rz : (rz > 1.0 ? 1.0 - rz : rz);
 
   /* get the point corresponding to r in the epsilon array grid: */
-  x = pmod(int(rx * nx), nx);
-  y = pmod(int(ry * ny), ny);
-  z = pmod(int(rz * nz), nz);
+  x = mirrorindex(int(rx * nx), nx);
+  y = mirrorindex(int(ry * ny), ny);
+  z = mirrorindex(int(rz * nz), nz);
 
-  /* get the difference between (x,y,z) and the actual point
-     ... we shift by 0.5 to center the data points in the pixels */
+  /* get the difference between (x,y,z) and the actual point */
   dx = rx * nx - x - 0.5;
   dy = ry * ny - y - 0.5;
   dz = rz * nz - z - 0.5;
 
   /* get the other closest point in the grid, with mirror boundaries: */
-  x2 = pmod((dx >= 0.0 ? x + 1 : x - 1), nx);
-  y2 = pmod((dy >= 0.0 ? y + 1 : y - 1), ny);
-  z2 = pmod((dz >= 0.0 ? z + 1 : z - 1), nz);
+  x2 = mirrorindex(dx >= 0.0 ? x + 1 : x - 1, nx);
+  y2 = mirrorindex(dy >= 0.0 ? y + 1 : y - 1, ny);
+  z2 = mirrorindex(dz >= 0.0 ? z + 1 : z - 1, nz);
 
   /* take abs(d{xyz}) to get weights for {xyz} and {xyz}2: */
   dx = fabs(dx);
