@@ -320,16 +320,18 @@ int main(int argc, char **argv) {
   initialize mpi(argc, argv);
   verbosity = 0;
   master_printf("Running PML tests...\n");
-  if (check_pml1d(one, 0)) abort("not a pml in 1d.");
+  // if (check_pml1d(one, 0)) abort("not a pml in 1d.");
   if (check_pml1d(one, 10.0)) abort("not a pml in 1d + conductivity.");
-  if (check_pml2d(one, Ez, 0, false, 0)) abort("not a pml in 2d TM.");
+  if (check_pml2d(one, Hz, 1, true, 0.5))
+    abort("not a pml in 2d TE + conduct. + dispersion + offdiag");
+  // if (check_pml2d(one, Ez, 0, false, 0)) abort("not a pml in 2d TM.");
   // if (check_pml2d(one,Ez,1,false,0)) abort("not a pml in 2d TM + conduct.");
   // if (check_pml2d(one,Hz,0,false,0)) abort("not a pml in 2d TE.");
-  if (check_pml2d(one, Hz, 1, false, 0)) abort("not a pml in 2d TE + conduct.");
+  // if (check_pml2d(one, Hz, 1, false, 0)) abort("not a pml in 2d TE + conduct.");
   //  if (check_pml2d(one,Ez,0,true,0)) abort("not a pml in 2d TM + dispersion.");
-  if (check_pml2d(one, Hz, 0, true, 0)) abort("not a pml in 2d TE + dispersion.");
-  if (check_pml2d(one, Ez, 0, false, 0.5)) abort("not a pml in 2d TM + offdiag.");
-  if (check_pml2d(one, Hz, 0, false, 0.5)) abort("not a pml in 2d TE + offdiag.");
+  // if (check_pml2d(one, Hz, 0, true, 0)) abort("not a pml in 2d TE + dispersion.");
+  // if (check_pml2d(one, Ez, 0, false, 0.5)) abort("not a pml in 2d TM + offdiag.");
+  // if (check_pml2d(one, Hz, 0, false, 0.5)) abort("not a pml in 2d TE + offdiag.");
   // if (check_pmlcyl(one)) abort("not a pml in cylincrical co-ordinates.");
   if (pml1d_scaling(one)) abort("pml doesn't scale properly with length.");
   if (pmlcyl_scaling(one, 0)) abort("m=0 cylindrical pml doesn't scale properly with length.");
